#  This file is part of Jellyfish.
#
#  This work is dual-licensed under 3-Clause BSD License or GPL 3.0.
#  You can choose between one of them if you use this work.
#
#  `SPDX-License-Identifier: BSD-3-Clause OR  GPL-3.0`

import unittest
import sys
import os
from collections import Counter

import dna_jellyfish as jf


class TestMerFile(unittest.TestCase):
    def setUp(self):
        self.mf = jf.ReadMerFile(os.path.join(data, "swig_python.jf"))

    def test_histo(self):
        histo = Counter()
        while self.mf.next_mer():
            histo[self.mf.count()] += 1

        jf_histo = Counter()
        with open(os.path.join(data, "swig_python.histo")) as f:
            for line in f:
                num, count = [int(n) for n in line.split()]
                self.assertEqual(count, histo[num])

    def test_dump(self):
        good = True
        with open(os.path.join(data, "swig_python.dump")) as f:
            for line in f:
                good = good and self.mf.next_mer()
                if not good: break
                a = line.split()
                good = good and a[0] == str(self.mf.mer()) and int(a[1]) == self.mf.count()
                if not good: break
        self.assertTrue(good)

    def test_iter(self):
        good = True
        with open(os.path.join(data, "swig_python.dump")) as f:
            for mer, count in self.mf:
                line = f.readline()
                good = good and line
                if not good: break
                fmer, fcount = line.split()
                good = good and fmer == str(mer) and int(fcount) == count
                if not good: break
            self.assertTrue(good)
            line = f.readline()
            self.assertTrue(not line)

    def test_query(self):
        good = True
        qf   = jf.QueryMerFile(os.path.join(data, "swig_python.jf"))
        for mer, count in self.mf:
            good = good and count == qf[mer]
            if not good: break
        self.assertTrue(good)

if __name__ == '__main__':
    data = sys.argv.pop(1)
    unittest.main()



#  This file is part of Jellyfish.
#
#  This work is dual-licensed under 3-Clause BSD License or GPL 3.0.
#  You can choose between one of them if you use this work.
#
#  `SPDX-License-Identifier: BSD-3-Clause OR  GPL-3.0`

import unittest
import sys
import random


import dna_jellyfish as jf

class TestHashCounter(unittest.TestCase):
    def setUp(self):
        jf.MerDNA.k(100)
        self.hash = jf.HashCounter(1024, 5)

    def test_info(self):
        self.assertEqual(100, jf.MerDNA.k())
        self.assertEqual(1024, self.hash.size())
        self.assertEqual(5, self.hash.val_len())

    def test_add(self):
        mer  = jf.MerDNA()
        good = True
        for i in range(1000):
            mer.randomize()
            val = random.randrange(1000)
            good = good and self.hash.add(mer, val)
            if not good: break
            if i % 3 > 0:
                nval = random.randrange(1000)
                val  = val + nval
                if i % 3 == 1:
                    good = good and (not self.hash.add(mer, nval))
                else:
                    good = good and self.hash.update_add(mer, nval)
            if not good: break
            good = good and (val == self.hash.get(mer)) and (val == self.hash[mer])
            if not good: break
        self.assertTrue(good)


if __name__ == '__main__':
    data = sys.argv.pop(1)
    unittest.main()



#! /usr/bin/env python
#  This file is part of Jellyfish.
#
#  This work is dual-licensed under 3-Clause BSD License or GPL 3.0.
#  You can choose between one of them if you use this work.
#
#  `SPDX-License-Identifier: BSD-3-Clause OR  GPL-3.0`

import os
import sys
import re
from distutils.core import setup, Extension

def pkgconfig(s):
    pipe = os.popen("pkg-config {}".format(s))
    res = pipe.read().rstrip().split()
    ret = pipe.close()
    if ret is not None:
        sys.stderr.write("\nCan't get compilation information for Jellyfish, check you PKG_CONFIG_PATH\n")
        exit(1)
    return res

jf_libs    = [re.sub(r'-l', '', x) for x in pkgconfig("--libs-only-l jellyfish-2.0")]
jf_include = [re.sub(r'-I', '', x) for x in pkgconfig("--cflags-only-I jellyfish-2.0")]
jf_cflags  = pkgconfig("--cflags-only-other jellyfish-2.0")
jf_libdir  = [re.sub(r'-L', '', x) for x in pkgconfig("--libs-only-L jellyfish-2.0")]
jf_rpath   = [re.sub(r'^', '-Wl,-rpath,', x) for x in jf_libdir]
jf_ldflags = pkgconfig("--libs-only-other jellyfish-2.0")

jellyfish_module = Extension('_dna_jellyfish',
                             sources = ['swig_wrap.cpp'],
                             include_dirs = jf_include,
                             libraries = jf_libs,
                             library_dirs = jf_libdir,
                             extra_compile_args = ["-std=c++0x"] + jf_cflags,
                             extra_link_args = jf_ldflags + jf_rpath,
                             language = "c++")
setup(name = 'dna_jellyfish',
      version = '0.0.1',
      author = 'Guillaume Marcais',
      description = 'Access to jellyfish k-mer counting',
      ext_modules = [jellyfish_module],
      py_modules = ["dna_jellyfish"])



#  This file is part of Jellyfish.
#
#  This work is dual-licensed under 3-Clause BSD License or GPL 3.0.
#  You can choose between one of them if you use this work.
#
#  `SPDX-License-Identifier: BSD-3-Clause OR  GPL-3.0`

import unittest
import sys
import random

import dna_jellyfish as jf

class TestStringMers(unittest.TestCase):
    def setUp(self):
        bases = "ACGTacgt"
        self.str = ''.join(random.choice(bases) for _ in range(1000))
        self.k = random.randint(10, 110)
        jf.MerDNA.k(self.k)

    def test_all_mers(self):
        count = 0
        good1 = True
        good2 = True
        mers = jf.string_mers(self.str)
        for m in mers:
            m2 = jf.MerDNA(self.str[count:count+self.k])
            good1 = good1 and m == m2
            good2 = good2 and self.str[count:count+self.k].upper() == str(m2)
            count += 1
        self.assertTrue(good1)
        self.assertTrue(good2)
        self.assertEqual(len(self.str) - self.k + 1, count)

    def test_canonical_mers(self):
        good = True
        mers = jf.string_canonicals(self.str)
        for count, m in enumerate(mers):
            m2 = jf.MerDNA(self.str[count:count+self.k])
            rm2 = m2.get_reverse_complement()
            good = good and (m == m2 or m == rm2)
            good = good and (not (m > m2)) and (not (m > rm2))
            # count += 1
        self.assertTrue(good)
        self.assertEqual(len(self.str) - self.k + 0, count)
        

if __name__ == '__main__':
    data = sys.argv.pop(1)
    unittest.main()



#  This file is part of Jellyfish.
#
#  This work is dual-licensed under 3-Clause BSD License or GPL 3.0.
#  You can choose between one of them if you use this work.
#
#  `SPDX-License-Identifier: BSD-3-Clause OR  GPL-3.0`

import warnings

warnings.warn("Module 'jellyfish' is deprecated. Use 'dna_jellyfish'", DeprecationWarning)
import dna_jellyfish as jellyfish
